/*
** iplog_udp.c - iplog UDP traffic logger.
** Copyright (C) 1999 behe <eric@ojnk.net>, Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <sys/types.h>

#include <pthread.h>

#include <iplog.h>
#include <iplog_filter.h>
#include <iplog_options.h>
#define DEF_ONLY
#include <iplog_scan.h>

#ifndef HAVE_UH_SPORT
#	define udphdr my_udp_header
struct my_udp_header {
	u_int16_t uh_sport;
	u_int16_t uh_dport;
	u_int16_t uh_ulen;
	u_int16_t uh_sum;
};
#endif

extern struct filter_data *filters[3];
extern u_long flags;

static struct udp_data {
	u_long t_host;
	time_t t_time;
} udp_data;

__inline__ bool udp_res(void) {
	return ((flags & UDP_RES) && !(flags & NO_RESOLV));
}

pthread_mutex_t udp_mutex = PTHREAD_MUTEX_INITIALIZER;

int udp_parser(struct ip *ip) {
	struct udphdr *udp = (struct udphdr *) ((char *) ip + (ip->ip_hl << 2));
	u_char buf[256], sbuf[256];
	u_long len;

	if (apply_filter(filters[FIL_UDP], ip, udp->uh_sport, udp->uh_dport))
		return (0);

	len = ntohs(ip->ip_len) - (ip->ip_hl << 2);

	if ((flags & SMURF) && ntohs(udp->uh_sport) == htons(7) &&
		check_scan(SCAN_SMURF, ip->ip_src.s_addr, len) != 0)
	{
		return (0);
	}

	if ((flags & TRACEROUTE) && ip->ip_ttl == 1 && ntohs(udp->uh_dport) > 30000)
	{
		pthread_mutex_lock(&udp_mutex);
		if (udp_data.t_host != ip->ip_src.s_addr ||
			(time(NULL) >= (udp_data.t_time + 15)))
		{
			udp_data.t_host = ip->ip_src.s_addr;
			time(&udp_data.t_time);
			mysyslog(PRIORITY, "UDP: traceroute from %s",
				host_lookup(&ip->ip_src, udp_res(), buf, sizeof(buf)));
		}
		pthread_mutex_unlock(&udp_mutex);

		return (0);
	}

	if ((flags & UDP_SCAN) && check_scan(SCAN_UDP, ip->ip_src.s_addr, len) != 0)
		return (0);

	mysyslog(PRIORITY, "UDP: dgram to %s from %s:%u (%lu data bytes)",
		serv_lookup(udp->uh_dport, "udp", sbuf, sizeof(sbuf)),
		host_lookup(&ip->ip_src, udp_res(), buf, sizeof(buf)),
		ntohs(udp->uh_sport), ntohs(udp->uh_ulen) - sizeof(*udp));

	return (0);
}

/* vim:ts=4:sw=8:tw=0 */
